/*
 * Empeg Logo file handler
 * Copyright (C) 2002 Toby Speight.  All rights reserved.
 *
 * The GIMP -- an image manipulation program
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * Based on xbm.c, X10 and X11 bitmap (XBM) loading and saving file filter for the GIMP.
 * XBM code Copyright (C) 1998 Gordon Matzigkeit.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* Set this for debugging. */
/* #define VERBOSE 2 */
#define VERBOSE 2

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "libgimp/gimp.h"

/* #define FALSE 0 */
/* #define TRUE 1 */

#define EMPEG_WIDTH 128
#define EMPEG_HEIGHT 64

#define MAKE_STR(x) MAKE_STR2(x)
#define MAKE_STR2(x) #x
#define EMPEG_WIDTH_STR MAKE_STR(EMPEG_WIDTH)
#define EMPEG_HEIGHT_STR MAKE_STR(EMPEG_HEIGHT)

typedef struct _EmpegSaveVals
{
  gchar tag[4];                 /* Empeg type identifier: "empg", "empc", "rio ", or "rioc" */
} EmpegSaveVals;

static EmpegSaveVals xsvals =
{
  "empc",                       /* tag */
};


typedef struct _EmpegSaveInterface
{
  gint run;
} EmpegSaveInterface;

static EmpegSaveInterface xsint =
{
  FALSE				/* run */
};


/* Declare some local functions.
 */
static void   query      (void);
static void   run        (char    *name,
                          int      nparams,
                          GParam  *param,
                          int     *nreturn_vals,
                          GParam **return_vals);
static gint32 load_image (char   *filename);
static gint   save_image (char   *filename,
			  gint32  image_ID,
			  gint32  drawable_ID);
#if 0
static gint   save_dialog (gint32  drawable_ID);
#endif
static void   close_callback  (GtkWidget *widget,
			       gpointer   data);
#if 0
static void   save_ok_callback     (GtkWidget *widget,
				    gpointer   data);
#endif
#if 0
static void   save_toggle_update   (GtkWidget *widget,
				    gpointer   data);
static void   comment_entry_callback (GtkWidget *widget,
				    gpointer   data);
static void   prefix_entry_callback(GtkWidget *widget,
				    gpointer   data);
#endif
static int    cmp_brightness (const void *a, const void *b);


GPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc */
};

MAIN ();

#ifdef VERBOSE
static int verbose = VERBOSE;
#endif


static void
query ()
{
  static GParamDef load_args[] =
  {
    { PARAM_INT32, "run_mode", "Interactive, non-interactive" },
    { PARAM_STRING, "filename", "The name of the file to load" },
    { PARAM_STRING, "raw_filename", "The name entered" },
  };
  static GParamDef load_return_vals[] =
  {
    { PARAM_IMAGE, "image", "Output image" },
  };
  static int nload_args = sizeof (load_args) / sizeof (load_args[0]);
  static int nload_return_vals = sizeof (load_return_vals) / sizeof (load_return_vals[0]);


  static GParamDef save_args[] =
  {
    { PARAM_INT32, "run_mode", "Interactive, non-interactive" },
    { PARAM_IMAGE, "image", "Input image" },
    { PARAM_DRAWABLE, "drawable", "Drawable to save" },
    { PARAM_STRING, "filename", "The name of the file to save" },
    { PARAM_STRING, "raw_filename", "The name entered" },
  } ;
  static int nsave_args = sizeof (save_args) / sizeof (save_args[0]);

  gimp_install_procedure ("file_empeg_load",
                          "Load a file in Empeg Logo file format",
                          "Load a file in Empeg Logo file format.  128x32 pixels, 4 grays.",
                          "Toby Speight",
                          "Toby Speight",
                          "2002",
                          "<Load>/Empeg Logo*",
			  NULL,
                          PROC_PLUG_IN,
                          nload_args, nload_return_vals,
                          load_args, load_return_vals);


  gimp_install_procedure ("file_empeg_save",
                          "Save a file in Empeg Logo file format",
                          "Save a file in Empeg Logo file format.  128x32 pixels, 4 grays.",
			  "Toby Speight",
                          "Toby Speight",
                          "2002",
                          "<Save>/Empeg Logo",
			  "INDEXED*",
                          PROC_PLUG_IN,
                          nsave_args, 0,
                          save_args, NULL);

  gimp_register_load_handler ("file_empeg_load", "empg,empeg", "");
  gimp_register_save_handler ("file_empeg_save", "empg,empeg", "");
}


static void
run (char    *name,
     int      nparams,
     GParam  *param,
     int     *nreturn_vals,
     GParam **return_vals)
{
  static GParam values[2];
  GStatusType status = STATUS_SUCCESS;
  GRunModeType run_mode;
  gint32 image_ID;

  run_mode = param[0].data.d_int32;

  *nreturn_vals = 1;
  *return_vals = values;
  values[0].type = PARAM_STATUS;
  values[0].data.d_status = STATUS_CALLING_ERROR;

#ifdef VERBOSE
  if (verbose)
    printf ("empeg: RUN %s\n", name);
#endif

  if (strcmp (name, "file_empeg_load") == 0)
    {
      image_ID = load_image (param[1].data.d_string);

      if (image_ID != -1)
        {
          *nreturn_vals = 2;
          values[0].data.d_status = STATUS_SUCCESS;
          values[1].type = PARAM_IMAGE;
          values[1].data.d_image = image_ID;
        }
      else
        {
          values[0].data.d_status = STATUS_EXECUTION_ERROR;
        }
    }
  else if (strcmp (name, "file_empeg_save") == 0)
    {
      int argc;
      char **argv;

      argc = 1;
      argv = g_new (gchar *, 1);
      argv[0] = g_strdup ("empeg");

      switch (run_mode)
	{
	case RUN_INTERACTIVE:
	  /*  Possibly retrieve data  */
	  gimp_get_data ("file_empeg_save", &xsvals);

	  /* Always override the prefix with the filename. */
/*  	  init_prefix (param[3].data.d_string); */

#if 0
	  /*  First acquire information with a dialog  */
	  if (! save_dialog (param[2].data.d_int32))
	    return;
#endif

	  break;

	case RUN_NONINTERACTIVE:
	  /*  Make sure all the required arguments are there!  */
	  if (nparams < 5)
	    status = STATUS_CALLING_ERROR;
	  if (status == STATUS_SUCCESS)
	    {
	      int i = 5;
#ifdef EMPEG_TYPE_SUPPORT
              // TODO:
	      if (nparams > i)
		{
		  memset (xsvals.comment, 0, sizeof (xsvals.comment));
		  strncpy (xsvals.comment, param[i].data.d_string,
			   MAX_COMMENT);
		}

	      i ++;
	      if (nparams > i)
		xsvals.x10_format = (param[i].data.d_int32) ? TRUE : FALSE;

	      i += 2;
	      if (nparams > i)
		{
		  /* They've asked for a hotspot. */
		  xsvals.x_hot = param[i - 1].data.d_int32;
		  xsvals.y_hot = param[i].data.d_int32;
		}

	      i ++;
	      if (nparams > i)
		{
		  memset (xsvals.prefix, 0, sizeof (xsvals.prefix));
		  strncpy (xsvals.prefix, param[i].data.d_string,
			   MAX_PREFIX);
		}
	      else
		init_prefix (param[3].data.d_string);
#endif
	      i ++;
	      /* Too many arguments. */
	      if (nparams > i)
		status = STATUS_CALLING_ERROR;
	    }
	  break;

	case RUN_WITH_LAST_VALS:
	  /*  Possibly retrieve data  */
	  gimp_get_data ("file_empeg_save", &xsvals);
	  break;

	default:
	  break;
	}

      *nreturn_vals = 1;
      if (save_image (param[3].data.d_string, param[1].data.d_int32,
		      param[2].data.d_int32))
	{
	  /*  Store xsvals data  */
	  gimp_set_data ("file_empeg_save", &xsvals, sizeof (xsvals));
	  values[0].data.d_status = STATUS_SUCCESS;
	}
      else
	values[0].data.d_status = STATUS_EXECUTION_ERROR;
    }
}


static gint
load_image (char *filename)
{
  FILE *fp;
  gint32 image_ID, layer_ID;

  GPixelRgn pixel_rgn;
  GDrawable *drawable;
  guchar *data;
  int c, i, j, k;
  int tileheight, rowoffset;

  char *name_buf;

  guchar cmap[] =
  {
    0x00, 0x00, 0x00,		/* black */
    0x20, 0x20, 0x20,
    0x80, 0x80, 0x80,
    0xff, 0xff, 0xff		/* white */
  };

  fp = fopen (filename, "rb");
  if (!fp)
    {
      printf ("Empeg: cannot open \"%s\"\n", filename);
      return -1;
    }

  /* TODO: store the type in a parasite - see gif.c */
  if (fseek(fp, 4, SEEK_CUR)) {
      printf ("Empeg: cannot skip Empeg header of \"%s\"\n", filename);
      return -1;
  }
    

  name_buf = g_malloc (strlen (filename) + 11);
  sprintf (name_buf, "Loading %s:", filename);
  gimp_progress_init (name_buf);
  g_free (name_buf);

  image_ID = gimp_image_new (EMPEG_WIDTH, EMPEG_HEIGHT, INDEXED);
  gimp_image_set_filename (image_ID, filename);

  /* Set the Empeg colormap. */
  gimp_image_set_cmap (image_ID, cmap, 4);

  layer_ID = gimp_layer_new (image_ID,
			     "Background",
			     EMPEG_WIDTH, EMPEG_HEIGHT,
			     INDEXED_IMAGE,
			     100,
			     NORMAL_MODE);
  gimp_image_add_layer (image_ID, layer_ID, 0);

  drawable = gimp_drawable_get (layer_ID);

  /* Prepare the pixel region. */
  gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0, EMPEG_WIDTH, EMPEG_HEIGHT, TRUE, FALSE);

  /* Allocate the data. */
  tileheight = gimp_tile_height ();
  data = (guchar *) g_malloc (EMPEG_WIDTH * tileheight);

  for (i = 0; i < EMPEG_HEIGHT; i += tileheight)
    {
      tileheight = MIN (tileheight, EMPEG_HEIGHT - i);

#ifdef VERBOSE
      if (verbose > 1)
	printf ("Empeg: reading %dx(%d+%d) pixel region\n", EMPEG_WIDTH, i,
		tileheight);
#endif

      /* Parse the data from the file */
      for (j = 0; j < tileheight; j ++)
	{
	  /* Read each row. */
	  rowoffset = j * EMPEG_WIDTH;
	  for (k = 0; k < EMPEG_WIDTH; k +=2)
	    {
                c = fgetc(fp);
                if (c < 0) {
                    printf("Empeg: unexpected EOF");
                    return -1;
                }
                data[rowoffset + k]     = c & 3;
                data[rowoffset + k + 1] = c >> 4;
	    }
	}

      /* Put the data into the image. */
      gimp_progress_update ((double) (i + tileheight) / (double) EMPEG_HEIGHT);
      gimp_pixel_rgn_set_rect (&pixel_rgn, data, 0, i, EMPEG_WIDTH, tileheight);
    }

  g_free (data);

  gimp_drawable_flush (drawable);
  gimp_drawable_detach (drawable);

  fclose (fp);
  return image_ID;
}


static void
close_callback (GtkWidget *widget,
		gpointer   data)
{
  gtk_main_quit ();
}

static int gtk_initialized = FALSE;


static void
error_dialog (char *msg)
{
  GtkWidget *dlg, *button, *label, *frame, *vbox;

  if (!gtk_initialized)
    {
      printf ("empeg: %s\n", msg);
      return;
    }

  dlg = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dlg), "Empeg Warning");
  gtk_window_position (GTK_WINDOW (dlg), GTK_WIN_POS_MOUSE);
  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
		      (GtkSignalFunc) close_callback,
		      dlg);
  gtk_signal_connect (GTK_OBJECT (dlg), "delete_event",
		      (GtkSignalFunc) close_callback,
		      dlg);

  /* Action area */
  button = gtk_button_new_with_label ("Cancel");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->action_area), button,
		      TRUE, TRUE, 0);
  gtk_widget_show (button);

      /*  the warning message  */
  frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_container_border_width (GTK_CONTAINER (frame), 10);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, TRUE,
		      TRUE, 0);
  vbox = gtk_vbox_new (FALSE, 5);
  gtk_container_border_width (GTK_CONTAINER (vbox), 5);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  label = gtk_label_new (msg);
  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
  gtk_widget_show (label);
  gtk_widget_show (vbox);
  gtk_widget_show (frame);
  gtk_widget_show (dlg);
  gtk_main ();
  gtk_widget_destroy (GTK_WIDGET (dlg));
  gdk_flush ();
}


static gint
save_image (char   *filename,
	    gint32  image_ID,
	    gint32  drawable_ID)
{
  GDrawable *drawable;
  GPixelRgn pixel_rgn;
  FILE *fp;
  
  int colors;
  int rowoffset, tileheight;
  int c, i, j, k;

  guchar *data, *cmap, *name_buf;
  guchar cmap_copy[16], out_map[4];

  drawable = gimp_drawable_get (drawable_ID);
  cmap = gimp_image_get_cmap (image_ID, &colors);

#if 0
  printf ("empeg: run `gdb empeg' and `attach %d'\n", getpid ());
  kill (getpid (), 19);
#endif

  if (drawable->width != EMPEG_WIDTH || drawable->height != EMPEG_HEIGHT)
    {
      /* The image is not the right size. */
      error_dialog (
                    "The image which you are trying to save as\n"
                    "an Empeg boot logo is the wrong size.\n\n"
                    "Please resize it to " EMPEG_WIDTH_STR "x" EMPEG_HEIGHT_STR " and try again."
                    );
      return FALSE;
    }

  if (gimp_drawable_type (drawable_ID) != INDEXED_IMAGE || colors > 4)
    {
      /* The image is not the right type. */
      error_dialog (
                    "The image which you are trying to save as\n"
                    "an Empeg logo contains more than four colors.\n\n"
                    "Please convert it to a four color\n"
                    "(2-bit) indexed image and try again."
                    );
      return FALSE;
    }

  name_buf = (guchar *) g_malloc (strlen (filename) + 11);
  sprintf (name_buf, "Saving %s:", filename);
  gimp_progress_init (name_buf);
  g_free (name_buf);

  {
    /* Sort the colors into ascending brightness. */
    for (i = 0;  i < colors;  i++) {
      cmap_copy[i*4] = cmap[i*3];
      cmap_copy[i*4+1] = cmap[i*3+1];
      cmap_copy[i*4+2] = cmap[i*3+2];
      cmap_copy[i*4+3] = i;
    }
    
    qsort (cmap_copy, colors, 4, cmp_brightness);
    
    i = 0;
    j = 0;
    out_map[i++] = cmap_copy[j++*4+3];
    if (colors > 3)
      out_map[i++] = cmap_copy[j++*4+3];
    if (colors > 2)
      out_map[i++] = cmap_copy[j++*4+3];
    if (colors > 1)             /* I can't imagine a 1-color logo, but... */
      out_map[i++] = cmap_copy[j++*4+3];

#ifdef VERBOSE
    if (verbose > 1)
      for (i = 0;  i < 4;  i++)
        printf("out_map[%d] = %d\n", i, out_map[i]);
#endif
  }


  /* Now actually save the data. */
  fp = fopen (filename, "w");
  if (!fp)
    {
      printf ("Empeg: cannot create \"%s\"\n", filename);
      return FALSE;
    }

  fputs("empg", fp);            /* TODO: make this configurable */

  /* Allocate a new set of pixels. */
  tileheight = gimp_tile_height ();
  data = (guchar *) g_malloc(EMPEG_WIDTH * tileheight);

  gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0, EMPEG_WIDTH, EMPEG_HEIGHT,
		       FALSE, FALSE);

  /* Write out the integers. */
  for (i = 0; i < EMPEG_HEIGHT; i += tileheight)
    {
      /* Get a horizontal slice of the image. */
      tileheight = MIN (tileheight, EMPEG_HEIGHT - i);
      gimp_pixel_rgn_get_rect (&pixel_rgn, data, 0, i, EMPEG_WIDTH, tileheight);

#ifdef VERBOSE
      if (verbose > 1)
	printf ("TGA: writing %dx(%d+%d) pixel region\n",
		EMPEG_WIDTH, i, tileheight);
#endif

      for (j = 0; j < tileheight; j ++)
	{
	  /* Write out a row at a time. */
	  rowoffset = j * EMPEG_WIDTH;
	  c = 0;

	  for (k = 0; k < EMPEG_WIDTH; k +=2)
	    {
                c = out_map[data[rowoffset + k]] | (out_map[data[rowoffset + k + 1]] << 4);
                fputc(c, fp);
	    }
	}

      gimp_progress_update ((double) (i + tileheight) / (double) EMPEG_HEIGHT);
    }

  fclose (fp);
  return TRUE;
}

static int
cmp_brightness (const void *a, const void *b) {
  const guchar *ca = a;
  const guchar *cb = b;
  /* TODO: is there a GIMP RGB->luminance function? */
  int la = 3*ca[0] + 6*ca[1] + ca[2];
  int lb = 3*cb[0] + 6*cb[1] + cb[2];
  return la - lb;
}


#if 0
static gint
save_dialog (gint32  drawable_ID)
{
  GtkWidget *dlg, *button, *frame, *vbox;
  gchar **argv;
  gint argc;

  xsint.run = FALSE;

  argc = 1;
  argv = g_new (gchar *, 1);
  argv[0] = g_strdup ("save");

  gtk_init (&argc, &argv);
  gtk_initialized = TRUE;

  gtk_rc_parse (gimp_gtkrc ());
  gdk_set_use_xshm(gimp_use_xshm());

  dlg = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (dlg), "Save as empeg");
  gtk_window_position (GTK_WINDOW (dlg), GTK_WIN_POS_MOUSE);
  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
		      (GtkSignalFunc) close_callback,
		      NULL);
  gtk_signal_connect (GTK_OBJECT (dlg), "delete_event",
		      (GtkSignalFunc) close_callback,
		      dlg);

  /*  Action area  */
  button = gtk_button_new_with_label ("OK");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      (GtkSignalFunc) save_ok_callback,
                      dlg);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->action_area), button,
		      TRUE, TRUE, 0);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);

  button = gtk_button_new_with_label ("Cancel");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			     (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT (dlg));
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->action_area), button,
		      TRUE, TRUE, 0);
  gtk_widget_show (button);


  /* parameter settings */
  frame = gtk_frame_new ("Empeg Options");
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  gtk_container_border_width (GTK_CONTAINER (frame), 10);
  gtk_widget_show (frame);

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, TRUE, TRUE, 0);
  vbox = gtk_vbox_new (FALSE, 5);
  gtk_container_border_width (GTK_CONTAINER (vbox), 5);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

#if 0
  /* comment string. */
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_border_width (GTK_CONTAINER (hbox), 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
  label = gtk_label_new ("Description: ");
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  entry = gtk_entry_new_with_max_length (MAX_COMMENT);
  gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
  gtk_widget_set_usize (entry, 240, 0);
  gtk_entry_set_text (GTK_ENTRY (entry), xsvals.comment);
  gtk_signal_connect (GTK_OBJECT (entry), "changed",
                      (GtkSignalFunc) comment_entry_callback,
                      NULL);
  gtk_widget_show (entry);
  gtk_widget_show (hbox);
#endif

#if 0
  /*  X10 format  */
  toggle = gtk_check_button_new_with_label ("X10 format bitmap");
  gtk_box_pack_start (GTK_BOX (vbox), toggle, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
		      (GtkSignalFunc) save_toggle_update,
		      &xsvals.x10_format);
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (toggle), xsvals.x10_format);
  gtk_widget_show (toggle);
#endif

#if 0
  /* prefix */
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_border_width (GTK_CONTAINER (hbox), 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

  label = gtk_label_new ("Identifier prefix: ");
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  entry = gtk_entry_new_with_max_length (MAX_PREFIX);
  gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
  gtk_entry_set_text (GTK_ENTRY (entry), xsvals.prefix);
  gtk_signal_connect (GTK_OBJECT (entry), "changed",
                      (GtkSignalFunc) prefix_entry_callback,
                      NULL);
  gtk_widget_show (entry);

  gtk_widget_show (hbox);
#endif

  /* Done. */
  gtk_widget_show (vbox);
  gtk_widget_show (dlg);

  gtk_main ();
  gdk_flush ();

  return xsint.run;
}
#endif


#if 0
/* Update the comment string. */
static void
comment_entry_callback  (GtkWidget *widget,
			 gpointer   data)
{
  memset (xsvals.comment, 0, sizeof (xsvals.comment));
  strncpy (xsvals.comment,
	   gtk_entry_get_text (GTK_ENTRY (widget)), MAX_COMMENT);
}


static void
prefix_entry_callback(GtkWidget *widget,
		      gpointer   data)
{
  memset (xsvals.prefix, 0, sizeof (xsvals.prefix));
  strncpy (xsvals.prefix, gtk_entry_get_text (GTK_ENTRY (widget)), MAX_PREFIX);
}
#endif

#if 0
static void
save_ok_callback (GtkWidget *widget,
		  gpointer   data)
{
  xsint.run = TRUE;
  gtk_widget_destroy (GTK_WIDGET (data));
}
#endif


#if 0
static void
save_toggle_update (GtkWidget *widget,
		    gpointer   data)
{
  int *toggle_val;

  toggle_val = (int *) data;

  if (GTK_TOGGLE_BUTTON (widget)->active)
    *toggle_val = TRUE;
  else
    *toggle_val = FALSE;
}
#endif

/*
Local Variables:
compile-command:"gcc -Wall -Wmissing-prototypes -g -O -o empeg empeg.c -I /usr/include/gtk-1.2 -I /usr/include/glib-1.2 -I /usr/lib/glib/include -lgimp -lgtk -lgdk -lglib -lm"
End:
*/
